# Generated by Django 3.2.9 on 2023-02-06 22:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('deleted_at', models.DateTimeField(blank=True, default=None, null=True, verbose_name='Deleted At')),
                ('title', models.CharField(blank=True, max_length=120, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RelatedTestModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('deleted_at', models.DateTimeField(blank=True, default=None, null=True, verbose_name='Deleted At')),
                ('title', models.CharField(blank=True, max_length=120, null=True)),
                ('test_model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='tests.testmodel')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
